@@ -47,31 +47,31 @@
  *
  */
 
-#include "miniserver.h"
+	#include "miniserver.h"
 
-#include "ThreadPool.h"
-#include "httpreadwrite.h"
-#include "ithread.h"
-#include "ssdplib.h"
-#include "statcodes.h"
-#include "unixutil.h" /* for socklen_t, EAFNOSUPPORT */
-#include "upnpapi.h"
-#include "upnputil.h"
+	#include "ThreadPool.h"
+	#include "httpreadwrite.h"
+	#include "ithread.h"
+	#include "ssdplib.h"
+	#include "statcodes.h"
+	#include "unixutil.h" /* for socklen_t, EAFNOSUPPORT */ // IWYU pragma: keep
+	#include "upnpapi.h"
+	#include "upnputil.h"
 
-#include <assert.h>
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
+	#include <assert.h>
+	#include <errno.h>
+	#include <stdio.h>
+	#include <stdlib.h>
+	#include <string.h>
+	#include <sys/types.h>
 
-/*! . */
-#define APPLICATION_LISTENING_PORT 49152
+	/*! . */
+	#define APPLICATION_LISTENING_PORT 49152
 
 struct mserv_request_t
 {
 	/*! Connection handle. */
-	SOCKET connfd;
+	SOCKET sock;
 	/*! . */
 	struct sockaddr_storage foreign_sockaddr;
 };
@@ -88,30 +88,44 @@ typedef enum
 } MiniServerState;
 
 /*! . */
-uint16_t miniStopSockPort;
+static uint16_t miniStopSockPort;
 
 /*!
  * module vars
  */
 static MiniServerState gMServState = MSERV_IDLE;
-#ifdef INTERNAL_WEB_SERVER
+
+/*! Global list to track active socket connections */
+static LinkedList gActiveConnections;
+static ithread_mutex_t gActiveConnectionsMutex;
+static int gActiveConnectionsInitialized = 0;
+
+/*! Structure to hold active connection info */
+struct active_connection_t
+{
+	SOCKET socket;
+	time_t connect_time;
+};
+
+/*----------------------------------------------------------------------------*/
+	#ifdef INTERNAL_WEB_SERVER
 static MiniServerCallback gGetCallback = NULL;
 static MiniServerCallback gSoapCallback = NULL;
 static MiniServerCallback gGenaCallback = NULL;
 
 static const int ENABLE_IPV6 =
-#ifdef UPNP_ENABLE_IPV6
+		#ifdef UPNP_ENABLE_IPV6
 	1;
-#else
+		#else  /* UPNP_ENABLE_IPV6 */
 	0;
-#endif
+		#endif /* UPNP_ENABLE_IPV6 */
 
 static const int MINISERVER_REUSEADDR =
-#ifdef UPNP_MINISERVER_REUSEADDR
+		#ifdef UPNP_MINISERVER_REUSEADDR
 	1;
-#else
+		#else  /* UPNP_MINISERVER_REUSEADDR */
 	0;
-#endif
+		#endif /* UPNP_MINISERVER_REUSEADDR */
 
 struct s_SocketStuff
 {
@@ -135,12 +149,93 @@ void SetHTTPGetCallback(MiniServerCallback callback)
 	gGetCallback = callback;
 }
 
-#ifdef INCLUDE_DEVICE_APIS
+		#ifdef INCLUDE_DEVICE_APIS
 void SetSoapCallback(MiniServerCallback callback) { gSoapCallback = callback; }
-#endif /* INCLUDE_DEVICE_APIS */
+		#endif /* INCLUDE_DEVICE_APIS */
 
 void SetGenaCallback(MiniServerCallback callback) { gGenaCallback = callback; }
 
+static int host_header_is_numeric(char *host_port, size_t host_port_size)
+{
+	int rc = 0;
+	struct in6_addr addr;
+	char *s;
+
+	/* Remove the port part. */
+	s = host_port + host_port_size - 1;
+	while (s != host_port && *s != ']' && *s != ':') {
+		--s;
+	}
+	if (*s == ':') {
+		*s = 0;
+	} else {
+		s = host_port + host_port_size;
+	}
+
+	/* Try IPV4 */
+	rc = inet_pton(AF_INET, host_port, &addr);
+	if (rc == 1) {
+		goto ExitFunction;
+	}
+	/* Try IPV6 */
+	/* Check for and remove the square brackets. */
+	if (strlen(host_port) < 3 || host_port[0] != '[' || *(s - 1) != ']') {
+		rc = 0;
+		goto ExitFunction;
+	}
+	*(s - 1) = '\0';
+	rc = inet_pton(AF_INET6, host_port + 1, &addr) == 1;
+
+ExitFunction:
+	return rc;
+}
+
+static int getNumericHostRedirection(
+	int socket, char *host_port, size_t hp_size)
+{
+	int rc;
+	const char *rcp;
+	struct sockaddr_storage addr;
+	struct sockaddr_in *addr4 = (struct sockaddr_in *)&addr;
+	struct sockaddr_in6 *addr6 = (struct sockaddr_in6 *)&addr;
+	socklen_t addr_len = sizeof addr;
+	in_port_t port;
+	char host[NAME_SIZE];
+	int n;
+
+	rc = getsockname(socket, (struct sockaddr *)&addr, &addr_len);
+	if (rc) {
+		goto ExitFunction;
+	}
+	switch (addr.ss_family) {
+	case AF_INET6:
+		rcp = inet_ntop(AF_INET6, &addr6->sin6_addr, host, sizeof host);
+		if (!rcp) {
+			rc = -1;
+			goto ExitFunction;
+		}
+		port = ntohs(addr6->sin6_port);
+		n = snprintf(host_port, hp_size, "[%s]:%d", host, port);
+		break;
+	case AF_INET:
+	default:
+		rcp = inet_ntop(AF_INET, &addr4->sin_addr, host, sizeof host);
+		if (!rcp) {
+			rc = -1;
+			goto ExitFunction;
+		}
+		port = ntohs(addr4->sin_port);
+		n = snprintf(host_port, hp_size, "%s:%d", host, port);
+		break;
+	}
+	if (n < 0) {
+		rc = -1;
+	}
+
+ExitFunction:
+	return rc == 0;
+}
+
 /*!
  * \brief Based on the type pf message, appropriate callback is issued.
  *
@@ -150,27 +245,41 @@ static int dispatch_request(
 	/*! [in] Socket Information object. */
 	SOCKINFO *info,
 	/*! [in] HTTP parser object. */
-	http_parser_t *hparser)
+	http_parser_t *parser)
 {
+	memptr header;
+	size_t min_size;
+	http_message_t *request;
 	MiniServerCallback callback;
+	WebCallback_HostValidate host_validate_callback = 0;
+	void *cookie = 0;
+	int rc = UPNP_E_SUCCESS;
+	/* If it does not fit in here, it is likely invalid anyway. */
+	char host_port[NAME_SIZE];
 
-	switch (hparser->msg.method) {
+	switch (parser->msg.method) {
 	/* Soap Call */
 	case SOAPMETHOD_POST:
 	case HTTPMETHOD_MPOST:
 		callback = gSoapCallback;
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"miniserver %d: got SOAP msg\n",
+			info->socket);
 		break;
 	/* Gena Call */
 	case HTTPMETHOD_NOTIFY:
 	case HTTPMETHOD_SUBSCRIBE:
 	case HTTPMETHOD_UNSUBSCRIBE:
+		callback = gGenaCallback;
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
 			__FILE__,
 			__LINE__,
 			"miniserver %d: got GENA msg\n",
 			info->socket);
-		callback = gGenaCallback;
 		break;
 	/* HTTP server call */
 	case HTTPMETHOD_GET:
@@ -178,16 +287,85 @@ static int dispatch_request(
 	case HTTPMETHOD_HEAD:
 	case HTTPMETHOD_SIMPLEGET:
 		callback = gGetCallback;
+		host_validate_callback = gWebCallback_HostValidate;
+		cookie = gWebCallback_HostValidateCookie;
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"miniserver %d: got WEB server msg\n",
+			info->socket);
 		break;
 	default:
-		callback = NULL;
+		callback = 0;
 	}
-	if (callback == NULL) {
-		return HTTP_INTERNAL_SERVER_ERROR;
+	if (!callback) {
+		rc = HTTP_INTERNAL_SERVER_ERROR;
+		goto ExitFunction;
 	}
-	callback(hparser, &hparser->msg, info);
+	request = &parser->msg;
+		#ifdef DEBUG_REDIRECT
+	/*--------------------------------------------------------------------*/
+	getNumericHostRedirection(info->socket, host_port, sizeof host_port);
+	UpnpPrintf(UPNP_INFO,
+		MSERV,
+		__FILE__,
+		__LINE__,
+		"DEBUG TEST: Redirect host_port = %s.\n",
+		host_port);
+		#endif /* DEBUG_REDIRECT */
+	/*--------------------------------------------------------------------*/
+	/* check HOST header for an IP number -- prevents DNS rebinding. */
+	if (!httpmsg_find_hdr(request, HDR_HOST, &header)) {
+		rc = UPNP_E_BAD_HTTPMSG;
+		goto ExitFunction;
+	}
+	min_size = header.length < ((sizeof host_port) - 1)
+			   ? header.length
+			   : (sizeof host_port) - 1;
+	memcpy(host_port, header.buf, min_size);
+	host_port[min_size] = 0;
+	if (host_validate_callback) {
+		rc = host_validate_callback(host_port, cookie);
+		if (rc == UPNP_E_BAD_HTTPMSG) {
+			goto ExitFunction;
+		}
+	} else if (!host_header_is_numeric(host_port, min_size)) {
+		if (!gAllowLiteralHostRedirection) {
+			rc = UPNP_E_BAD_HTTPMSG;
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"Possible DNS Rebind attack prevented.\n");
+			goto ExitFunction;
+		} else {
+			membuffer redir_buf;
+			char redir_str[2 * NAME_SIZE];
+			int timeout = HTTP_DEFAULT_TIMEOUT;
 
-	return 0;
+			getNumericHostRedirection(
+				(int)info->socket, host_port, sizeof host_port);
+			membuffer_init(&redir_buf);
+			snprintf(redir_str,
+				sizeof redir_str,
+				"HTTP/1.1 307 Temporary Redirect\r\n"
+				"Location: http://%s\r\n\r\n",
+				host_port);
+			membuffer_append_str(&redir_buf, redir_str);
+			rc = http_SendMessage(info,
+				&timeout,
+				"b",
+				redir_buf.buf,
+				redir_buf.length);
+			membuffer_destroy(&redir_buf);
+			goto ExitFunction;
+		}
+	}
+	callback(parser, request, info);
+
+ExitFunction:
+	return rc;
 }
 
 /*!
@@ -207,7 +385,81 @@ static UPNP_INLINE void handle_error(
 }
 
 /*!
- * \brief Free memory assigned for handling request and unitialize socket
+ * \brief Compare function for active connections, return true if equal (found)
+ */
+static int active_connection_cmp(void *first, void *second)
+{
+	struct active_connection_t *conn1 = (struct active_connection_t *)first;
+	struct active_connection_t *conn2 =
+		(struct active_connection_t *)second;
+
+	/* Return non-zero (true) if sockets match */
+	return (conn1->socket == conn2->socket);
+}
+
+/*!
+ * \brief Add a socket to the active connections list
+ */
+static void add_active_connection(SOCKET sock)
+{
+	struct active_connection_t *conn;
+
+	if (!gActiveConnectionsInitialized) {
+		ListInit(&gActiveConnections, active_connection_cmp, free);
+		ithread_mutex_init(&gActiveConnectionsMutex, NULL);
+		gActiveConnectionsInitialized = 1;
+	}
+
+	conn = (struct active_connection_t *)malloc(
+		sizeof(struct active_connection_t));
+	if (conn) {
+		conn->socket = sock;
+		conn->connect_time = time(NULL);
+
+		ithread_mutex_lock(&gActiveConnectionsMutex);
+		ListAddTail(&gActiveConnections, conn);
+		ithread_mutex_unlock(&gActiveConnectionsMutex);
+
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"Added active connection: socket %d\n",
+			sock);
+	}
+}
+
+/*!
+ * \brief Remove a socket from the active connections list
+ */
+static void remove_active_connection(SOCKET sock)
+{
+	struct active_connection_t search_conn;
+	ListNode *node;
+
+	if (!gActiveConnectionsInitialized) {
+		return;
+	}
+	/* Create a temporary connection structure for searching */
+	search_conn.socket = sock;
+	search_conn.connect_time = 0; /* Not used for comparison */
+
+	ithread_mutex_lock(&gActiveConnectionsMutex);
+	node = ListFind(&gActiveConnections, NULL, &search_conn);
+	if (node) {
+		ListDelNode(&gActiveConnections, node, 1);
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"Removed active connection: socket %d\n",
+			sock);
+	}
+	ithread_mutex_unlock(&gActiveConnectionsMutex);
+}
+
+/*!
+ * \brief Free memory assigned for handling request and uninitialize socket
  * functionality.
  */
 static void free_handle_request_arg(
@@ -216,7 +468,8 @@ static void free_handle_request_arg(
 {
 	struct mserv_request_t *request = (struct mserv_request_t *)args;
 
-	sock_close(request->connfd);
+	remove_active_connection(request->sock);
+	sock_close(request->sock);
 	free(request);
 }
 
@@ -233,24 +486,24 @@ static void handle_request(
 	int major = 1;
 	int minor = 1;
 	http_parser_t parser;
-	http_message_t *hmsg = NULL;
+	http_message_t *h_msg = NULL;
 	int timeout = HTTP_DEFAULT_TIMEOUT;
 	struct mserv_request_t *request = (struct mserv_request_t *)args;
-	SOCKET connfd = request->connfd;
+	SOCKET sock = request->sock;
 
 	UpnpPrintf(UPNP_INFO,
 		MSERV,
 		__FILE__,
 		__LINE__,
 		"miniserver %d: READING\n",
-		connfd);
+		sock);
 	/* parser_request_init( &parser ); */ /* LEAK_FIX_MK */
-	hmsg = &parser.msg;
+	h_msg = &parser.msg;
 	ret_code = sock_init_with_ip(
-		&info, connfd, (struct sockaddr *)&request->foreign_sockaddr);
+		&info, sock, (struct sockaddr *)&request->foreign_sockaddr);
 	if (ret_code != UPNP_E_SUCCESS) {
 		free(request);
-		httpmsg_destroy(hmsg);
+		httpmsg_destroy(h_msg);
 		return;
 	}
 	/* read */
@@ -264,7 +517,7 @@ static void handle_request(
 		__FILE__,
 		__LINE__,
 		"miniserver %d: PROCESSING...\n",
-		connfd);
+		sock);
 	/* dispatch */
 	http_error_code = dispatch_request(&info, &parser);
 	if (http_error_code != 0) {
@@ -274,14 +527,14 @@ static void handle_request(
 
 error_handler:
 	if (http_error_code > 0) {
-		if (hmsg) {
-			major = hmsg->major_version;
-			minor = hmsg->minor_version;
+		if (h_msg) {
+			major = h_msg->major_version;
+			minor = h_msg->minor_version;
 		}
 		handle_error(&info, http_error_code, major, minor);
 	}
 	sock_destroy(&info, SD_BOTH);
-	httpmsg_destroy(hmsg);
+	httpmsg_destroy(h_msg);
 	free(request);
 
 	UpnpPrintf(UPNP_INFO,
@@ -289,16 +542,16 @@ error_handler:
 		__FILE__,
 		__LINE__,
 		"miniserver %d: COMPLETE\n",
-		connfd);
+		sock);
 }
 
 /*!
- * \brief Initilize the thread pool to handle a request, sets priority for the
+ * \brief Initialize the thread pool to handle a request, sets priority for the
  * job and adds the job to the thread pool.
  */
 static UPNP_INLINE void schedule_request_job(
 	/*! [in] Socket Descriptor on which connection is accepted. */
-	SOCKET connfd,
+	SOCKET sock,
 	/*! [in] Clients Address information. */
 	struct sockaddr *clientAddr)
 {
@@ -309,37 +562,97 @@ static UPNP_INLINE void schedule_request_job(
 
 	request = (struct mserv_request_t *)malloc(
 		sizeof(struct mserv_request_t));
-	if (request == NULL) {
+	if (!request) {
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
 			__FILE__,
 			__LINE__,
 			"mserv %d: out of memory\n",
-			connfd);
-		sock_close(connfd);
+			sock);
+		sock_close(sock);
 		return;
 	}
 
-	request->connfd = connfd;
+	request->sock = sock;
 	memcpy(&request->foreign_sockaddr,
 		clientAddr,
 		sizeof(request->foreign_sockaddr));
 	TPJobInit(&job, (start_routine)handle_request, (void *)request);
 	TPJobSetFreeFunction(&job, free_handle_request_arg);
 	TPJobSetPriority(&job, MED_PRIORITY);
+
+	/* Add the connection to active connections list */
+	add_active_connection(sock);
+
 	if (ThreadPoolAdd(&gMiniServerThreadPool, &job, NULL) != 0) {
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
 			__FILE__,
 			__LINE__,
 			"mserv %d: cannot schedule request\n",
-			connfd);
+			sock);
+		remove_active_connection(sock);
 		free(request);
-		sock_close(connfd);
+		sock_close(sock);
 		return;
 	}
 }
-#endif
+	#endif /* INTERNAL_WEB_SERVER */
+/*--------------------------------------------------------------------*/
+
+/*!
+ * \brief Shutdown all active socket connections
+ */
+void shutdown_all_active_connections(void)
+{
+	/*--------------------------------------------------------------------*/
+	#ifdef INTERNAL_WEB_SERVER
+	ListNode *node;
+	int count;
+
+	if (!gActiveConnectionsInitialized) {
+		return;
+	}
+	UpnpPrintf(UPNP_INFO,
+		MSERV,
+		__FILE__,
+		__LINE__,
+		"Shutting down all active socket connections\n");
+
+	ithread_mutex_lock(&gActiveConnectionsMutex);
+	node = ListHead(&gActiveConnections);
+	count = 0;
+	while (node) {
+		struct active_connection_t *conn =
+			(struct active_connection_t *)node->item;
+		if (conn && conn->socket != INVALID_SOCKET) {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"Force shutting down socket %d\n",
+				conn->socket);
+			shutdown(conn->socket, SD_BOTH);
+			count++;
+		}
+		node = ListNext(&gActiveConnections, node);
+	}
+
+	/* Destroy the list and reset state */
+	ListDestroy(&gActiveConnections, 1);
+	gActiveConnectionsInitialized = 0;
+	ithread_mutex_unlock(&gActiveConnectionsMutex);
+	ithread_mutex_destroy(&gActiveConnectionsMutex);
+
+	UpnpPrintf(UPNP_INFO,
+		MSERV,
+		__FILE__,
+		__LINE__,
+		"Shutdown %d active socket connections and clean\n",
+		count);
+	#endif /* INTERNAL_WEB_SERVER */
+	/*--------------------------------------------------------------------*/
+}
 
 static UPNP_INLINE void fdset_if_valid(SOCKET sock, fd_set *set)
 {
@@ -348,18 +661,20 @@ static UPNP_INLINE void fdset_if_valid(SOCKET sock, fd_set *set)
 	}
 }
 
-static void web_server_accept(SOCKET lsock, fd_set *set)
+static void web_server_accept(SOCKET listen_sock, fd_set *set)
 {
-#ifdef INTERNAL_WEB_SERVER
+	/*--------------------------------------------------------------------*/
+	#ifdef INTERNAL_WEB_SERVER
 	SOCKET asock;
 	socklen_t clientLen;
 	struct sockaddr_storage clientAddr;
 	char errorBuffer[ERROR_BUFFER_LEN];
 
-	if (lsock != INVALID_SOCKET && FD_ISSET(lsock, set)) {
+	if (listen_sock != INVALID_SOCKET && FD_ISSET(listen_sock, set)) {
 		clientLen = sizeof(clientAddr);
-		asock = accept(
-			lsock, (struct sockaddr *)&clientAddr, &clientLen);
+		asock = accept(listen_sock,
+			(struct sockaddr *)&clientAddr,
+			&clientLen);
 		if (asock == INVALID_SOCKET) {
 			strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
 			UpnpPrintf(UPNP_INFO,
@@ -373,13 +688,25 @@ static void web_server_accept(SOCKET lsock, fd_set *set)
 				asock, (struct sockaddr *)&clientAddr);
 		}
 	}
-#endif /* INTERNAL_WEB_SERVER */
+	#endif /* INTERNAL_WEB_SERVER */
+	/*--------------------------------------------------------------------*/
 }
 
-static void ssdp_read(SOCKET rsock, fd_set *set)
+static void ssdp_read(SOCKET *read_sock, fd_set *set)
 {
-	if (rsock != INVALID_SOCKET && FD_ISSET(rsock, set)) {
-		readFromSSDPSocket(rsock);
+	if (*read_sock != INVALID_SOCKET && FD_ISSET(*read_sock, set)) {
+		int ret = readFromSSDPSocket(*read_sock);
+		if (ret != 0) {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"miniserver: Error in readFromSSDPSocket(%d): "
+				"closing socket\n",
+				*read_sock);
+			sock_close(*read_sock);
+			*read_sock = INVALID_SOCKET;
+		}
 	}
 }
 
@@ -422,6 +749,13 @@ static int receive_from_stopSock(SOCKET ssock, fd_set *set)
 			if (NULL != strstr(requestBuf, "ShutDown")) {
 				return 1;
 			}
+		} else {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"miniserver: stopSock Error, aborting...\n");
+			return 1;
 		}
 	}
 
@@ -437,7 +771,7 @@ static int receive_from_stopSock(SOCKET ssock, fd_set *set)
  */
 static void RunMiniServer(
 	/*! [in] Socket Array. */
-	MiniServerSockArray *miniSock)
+	MiniServerSockArray *mini_sock)
 {
 	char errorBuffer[ERROR_BUFFER_LEN];
 	fd_set expSet;
@@ -447,17 +781,17 @@ static void RunMiniServer(
 	int stopSock = 0;
 
 	maxMiniSock = 0;
-	maxMiniSock = max(maxMiniSock, miniSock->miniServerSock4);
-	maxMiniSock = max(maxMiniSock, miniSock->miniServerSock6);
-	maxMiniSock = max(maxMiniSock, miniSock->miniServerSock6UlaGua);
-	maxMiniSock = max(maxMiniSock, miniSock->miniServerStopSock);
-	maxMiniSock = max(maxMiniSock, miniSock->ssdpSock4);
-	maxMiniSock = max(maxMiniSock, miniSock->ssdpSock6);
-	maxMiniSock = max(maxMiniSock, miniSock->ssdpSock6UlaGua);
-#ifdef INCLUDE_CLIENT_APIS
-	maxMiniSock = max(maxMiniSock, miniSock->ssdpReqSock4);
-	maxMiniSock = max(maxMiniSock, miniSock->ssdpReqSock6);
-#endif /* INCLUDE_CLIENT_APIS */
+	maxMiniSock = max(maxMiniSock, mini_sock->miniServerSock4);
+	maxMiniSock = max(maxMiniSock, mini_sock->miniServerSock6);
+	maxMiniSock = max(maxMiniSock, mini_sock->miniServerSock6UlaGua);
+	maxMiniSock = max(maxMiniSock, mini_sock->miniServerStopSock);
+	maxMiniSock = max(maxMiniSock, mini_sock->ssdpSock4);
+	maxMiniSock = max(maxMiniSock, mini_sock->ssdpSock6);
+	maxMiniSock = max(maxMiniSock, mini_sock->ssdpSock6UlaGua);
+	#ifdef INCLUDE_CLIENT_APIS
+	maxMiniSock = max(maxMiniSock, mini_sock->ssdpReqSock4);
+	maxMiniSock = max(maxMiniSock, mini_sock->ssdpReqSock6);
+	#endif /* INCLUDE_CLIENT_APIS */
 	++maxMiniSock;
 
 	gMServState = MSERV_RUNNING;
@@ -465,18 +799,18 @@ static void RunMiniServer(
 		FD_ZERO(&rdSet);
 		FD_ZERO(&expSet);
 		/* FD_SET()'s */
-		FD_SET(miniSock->miniServerStopSock, &expSet);
-		FD_SET(miniSock->miniServerStopSock, &rdSet);
-		fdset_if_valid(miniSock->miniServerSock4, &rdSet);
-		fdset_if_valid(miniSock->miniServerSock6, &rdSet);
-		fdset_if_valid(miniSock->miniServerSock6UlaGua, &rdSet);
-		fdset_if_valid(miniSock->ssdpSock4, &rdSet);
-		fdset_if_valid(miniSock->ssdpSock6, &rdSet);
-		fdset_if_valid(miniSock->ssdpSock6UlaGua, &rdSet);
-#ifdef INCLUDE_CLIENT_APIS
-		fdset_if_valid(miniSock->ssdpReqSock4, &rdSet);
-		fdset_if_valid(miniSock->ssdpReqSock6, &rdSet);
-#endif /* INCLUDE_CLIENT_APIS */
+		FD_SET(mini_sock->miniServerStopSock, &expSet);
+		FD_SET(mini_sock->miniServerStopSock, &rdSet);
+		fdset_if_valid(mini_sock->miniServerSock4, &rdSet);
+		fdset_if_valid(mini_sock->miniServerSock6, &rdSet);
+		fdset_if_valid(mini_sock->miniServerSock6UlaGua, &rdSet);
+		fdset_if_valid(mini_sock->ssdpSock4, &rdSet);
+		fdset_if_valid(mini_sock->ssdpSock6, &rdSet);
+		fdset_if_valid(mini_sock->ssdpSock6UlaGua, &rdSet);
+	#ifdef INCLUDE_CLIENT_APIS
+		fdset_if_valid(mini_sock->ssdpReqSock4, &rdSet);
+		fdset_if_valid(mini_sock->ssdpReqSock6, &rdSet);
+	#endif /* INCLUDE_CLIENT_APIS */
 		/* select() */
 		ret = select((int)maxMiniSock, &rdSet, NULL, &expSet, NULL);
 		if (ret == SOCKET_ERROR && errno == EINTR) {
@@ -492,35 +826,39 @@ static void RunMiniServer(
 				errorBuffer);
 			continue;
 		} else {
-			web_server_accept(miniSock->miniServerSock4, &rdSet);
-			web_server_accept(miniSock->miniServerSock6, &rdSet);
+			web_server_accept(mini_sock->miniServerSock4, &rdSet);
+			web_server_accept(mini_sock->miniServerSock6, &rdSet);
 			web_server_accept(
-				miniSock->miniServerSock6UlaGua, &rdSet);
-#ifdef INCLUDE_CLIENT_APIS
-			ssdp_read(miniSock->ssdpReqSock4, &rdSet);
-			ssdp_read(miniSock->ssdpReqSock6, &rdSet);
-#endif /* INCLUDE_CLIENT_APIS */
-			ssdp_read(miniSock->ssdpSock4, &rdSet);
-			ssdp_read(miniSock->ssdpSock6, &rdSet);
-			ssdp_read(miniSock->ssdpSock6UlaGua, &rdSet);
+				mini_sock->miniServerSock6UlaGua, &rdSet);
+	#ifdef INCLUDE_CLIENT_APIS
+			ssdp_read(&mini_sock->ssdpReqSock4, &rdSet);
+			ssdp_read(&mini_sock->ssdpReqSock6, &rdSet);
+	#endif /* INCLUDE_CLIENT_APIS */
+			ssdp_read(&mini_sock->ssdpSock4, &rdSet);
+			ssdp_read(&mini_sock->ssdpSock6, &rdSet);
+			ssdp_read(&mini_sock->ssdpSock6UlaGua, &rdSet);
 			stopSock = receive_from_stopSock(
-				miniSock->miniServerStopSock, &rdSet);
+				mini_sock->miniServerStopSock, &rdSet);
 		}
 	}
+
+	/* shutdown connections */
+	shutdown_all_active_connections();
+
 	/* Close all sockets. */
-	sock_close(miniSock->miniServerSock4);
-	sock_close(miniSock->miniServerSock6);
-	sock_close(miniSock->miniServerSock6UlaGua);
-	sock_close(miniSock->miniServerStopSock);
-	sock_close(miniSock->ssdpSock4);
-	sock_close(miniSock->ssdpSock6);
-	sock_close(miniSock->ssdpSock6UlaGua);
-#ifdef INCLUDE_CLIENT_APIS
-	sock_close(miniSock->ssdpReqSock4);
-	sock_close(miniSock->ssdpReqSock6);
-#endif /* INCLUDE_CLIENT_APIS */
-	/* Free minisock. */
-	free(miniSock);
+	sock_close(mini_sock->miniServerSock4);
+	sock_close(mini_sock->miniServerSock6);
+	sock_close(mini_sock->miniServerSock6UlaGua);
+	sock_close(mini_sock->miniServerStopSock);
+	sock_close(mini_sock->ssdpSock4);
+	sock_close(mini_sock->ssdpSock6);
+	sock_close(mini_sock->ssdpSock6UlaGua);
+	#ifdef INCLUDE_CLIENT_APIS
+	sock_close(mini_sock->ssdpReqSock4);
+	sock_close(mini_sock->ssdpReqSock6);
+	#endif /* INCLUDE_CLIENT_APIS */
+	/* Free mini_sock. */
+	free(mini_sock);
 	gMServState = MSERV_IDLE;
 
 	return;
@@ -529,7 +867,7 @@ static void RunMiniServer(
 /*!
  * \brief Returns port to which socket, sockfd, is bound.
  *
- * \return -1 on error; check errno. 0 if successfull.
+ * \return -1 on error; check errno. 0 if successful.
  */
 static int get_port(
 	/*! [in] Socket descriptor. */
@@ -562,8 +900,8 @@ static int get_port(
 	return 0;
 }
 
-#ifdef INTERNAL_WEB_SERVER
-static int init_socket_suff(
+	#ifdef INTERNAL_WEB_SERVER
+static int init_socket_stuff(
 	struct s_SocketStuff *s, const char *text_addr, int ip_version)
 {
 	char errorBuffer[ERROR_BUFFER_LEN];
@@ -598,12 +936,14 @@ static int init_socket_suff(
 			MSERV,
 			__FILE__,
 			__LINE__,
-			"init_socket_suff(): Invalid IP version: %d.\n",
+			"init_socket_stuff(): Invalid IP version: %d.\n",
 			ip_version);
 		goto error;
-		break;
 	}
-	inet_pton(domain, text_addr, addr);
+
+	if (inet_pton(domain, text_addr, addr) <= 0)
+		goto error;
+
 	s->fd = socket(domain, SOCK_STREAM, 0);
 	if (s->fd == INVALID_SOCKET) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -611,7 +951,7 @@ static int init_socket_suff(
 			MSERV,
 			__FILE__,
 			__LINE__,
-			"init_socket_suff(): IPv%c socket not available: "
+			"init_socket_stuff(): IPv%c socket not available: "
 			"%s\n",
 			ip_version,
 			errorBuffer);
@@ -630,7 +970,7 @@ static int init_socket_suff(
 				MSERV,
 				__FILE__,
 				__LINE__,
-				"init_socket_suff(): unable to set IPv6 "
+				"init_socket_stuff(): unable to set IPv6 "
 				"socket protocol: %s\n",
 				errorBuffer);
 			goto error;
@@ -653,7 +993,7 @@ static int init_socket_suff(
 				MSERV,
 				__FILE__,
 				__LINE__,
-				"init_socket_suff(): unable to set "
+				"init_socket_stuff(): unable to set "
 				"SO_REUSEADDR: %s\n",
 				errorBuffer);
 			goto error;
@@ -694,11 +1034,11 @@ static int do_bind(struct s_SocketStuff *s)
 		}
 		bind_error = bind(s->fd, s->serverAddr, s->address_len);
 		if (bind_error == SOCKET_ERROR) {
-#ifdef _WIN32
+		#ifdef _WIN32
 			errCode = WSAGetLastError();
-#else
+		#else  /* _WIN32 */
 			errCode = errno;
-#endif
+		#endif /* _WIN32 */
 			if (errno == EADDRINUSE) {
 				errCode = 1;
 			}
@@ -716,7 +1056,7 @@ static int do_bind(struct s_SocketStuff *s)
 			"Error in IPv%d bind(): %s\n",
 			s->ip_version,
 			errorBuffer);
-		/* Bind failied. */
+		/* Bind failed. */
 		ret_val = UPNP_E_SOCKET_BIND;
 		goto error;
 	}
@@ -768,7 +1108,7 @@ static int do_reinit(struct s_SocketStuff *s)
 {
 	sock_close(s->fd);
 
-	return init_socket_suff(s, s->text_addr, s->ip_version);
+	return init_socket_stuff(s, s->text_addr, s->ip_version);
 }
 
 static int do_bind_listen(struct s_SocketStuff *s)
@@ -801,7 +1141,7 @@ error:
 
 /*!
  * \brief Creates a STREAM socket, binds to INADDR_ANY and listens for
- * incoming connecttions. Returns the actual port which the sockets
+ * incoming connections. Returns the actual port which the sockets
  * sub-system returned.
  *
  * Also creates a DGRAM socket, binds to the loop back address and
@@ -838,9 +1178,9 @@ static int get_miniserver_sockets(
 
 	/* Create listen socket for IPv4/IPv6. An error here may indicate
 	 * that we don't have an IPv4/IPv6 stack. */
-	err_init_4 = init_socket_suff(&ss4, gIF_IPV4, 4);
-	err_init_6 = init_socket_suff(&ss6, gIF_IPV6, 6);
-	err_init_6UlaGua = init_socket_suff(&ss6UlaGua, gIF_IPV6_ULA_GUA, 6);
+	err_init_4 = init_socket_stuff(&ss4, gIF_IPV4, 4);
+	err_init_6 = init_socket_stuff(&ss6, gIF_IPV6, 6);
+	err_init_6UlaGua = init_socket_stuff(&ss6UlaGua, gIF_IPV6_ULA_GUA, 6);
 	ss6.serverAddr6->sin6_scope_id = gIF_INDEX;
 	/* Check what happened. */
 	if (err_init_4 && (err_init_6 || err_init_6UlaGua)) {
@@ -871,7 +1211,7 @@ static int get_miniserver_sockets(
 		/* THIS IS ALLOWS US TO BIND AGAIN IMMEDIATELY
 		 * AFTER OUR SERVER HAS BEEN CLOSED
 		 * THIS MAY CAUSE TCP TO BECOME LESS RELIABLE
-		 * HOWEVER IT HAS BEEN SUGESTED FOR TCP SERVERS. */
+		 * HOWEVER IT HAS BEEN SUGGESTED FOR TCP SERVERS. */
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
 			__FILE__,
@@ -923,7 +1263,7 @@ error:
 
 	return ret_val;
 }
-#endif /* INTERNAL_WEB_SERVER */
+	#endif /* INTERNAL_WEB_SERVER */
 
 /*!
  * \brief Creates the miniserver STOP socket. This socket is created and
@@ -958,7 +1298,7 @@ static int get_miniserver_stopsock(
 	/* Bind to local socket. */
 	memset(&stop_sockaddr, 0, sizeof(stop_sockaddr));
 	stop_sockaddr.sin_family = (sa_family_t)AF_INET;
-	stop_sockaddr.sin_addr.s_addr = inet_addr("127.0.0.1");
+	inet_pton(AF_INET, "127.0.0.1", &stop_sockaddr.sin_addr);
 	ret = bind(miniServerStopSock,
 		(struct sockaddr *)&stop_sockaddr,
 		sizeof(stop_sockaddr));
@@ -995,10 +1335,10 @@ static UPNP_INLINE void InitMiniServerSockArray(MiniServerSockArray *miniSocket)
 	miniSocket->miniServerPort4 = 0u;
 	miniSocket->miniServerPort6 = 0u;
 	miniSocket->miniServerPort6UlaGua = 0u;
-#ifdef INCLUDE_CLIENT_APIS
+	#ifdef INCLUDE_CLIENT_APIS
 	miniSocket->ssdpReqSock4 = INVALID_SOCKET;
 	miniSocket->ssdpReqSock6 = INVALID_SOCKET;
-#endif /* INCLUDE_CLIENT_APIS */
+	#endif /* INCLUDE_CLIENT_APIS */
 }
 
 int StartMiniServer(
@@ -1032,7 +1372,7 @@ int StartMiniServer(
 		return UPNP_E_OUTOF_MEMORY;
 	}
 	InitMiniServerSockArray(miniSocket);
-#ifdef INTERNAL_WEB_SERVER
+	#ifdef INTERNAL_WEB_SERVER
 	/* V4 and V6 http listeners. */
 	ret_code = get_miniserver_sockets(
 		miniSocket, *listen_port4, *listen_port6, *listen_port6UlaGua);
@@ -1040,7 +1380,7 @@ int StartMiniServer(
 		free(miniSocket);
 		return ret_code;
 	}
-#endif
+	#endif /* INTERNAL_WEB_SERVER */
 	/* Stop socket (To end miniserver processing). */
 	ret_code = get_miniserver_stopsock(miniSocket);
 	if (ret_code != UPNP_E_SUCCESS) {
@@ -1072,10 +1412,10 @@ int StartMiniServer(
 		sock_close(miniSocket->ssdpSock4);
 		sock_close(miniSocket->ssdpSock6);
 		sock_close(miniSocket->ssdpSock6UlaGua);
-#ifdef INCLUDE_CLIENT_APIS
+	#ifdef INCLUDE_CLIENT_APIS
 		sock_close(miniSocket->ssdpReqSock4);
 		sock_close(miniSocket->ssdpReqSock6);
-#endif /* INCLUDE_CLIENT_APIS */
+	#endif /* INCLUDE_CLIENT_APIS */
 		free(miniSocket);
 		return UPNP_E_OUTOF_MEMORY;
 	}
@@ -1096,22 +1436,22 @@ int StartMiniServer(
 		sock_close(miniSocket->ssdpSock4);
 		sock_close(miniSocket->ssdpSock6);
 		sock_close(miniSocket->ssdpSock6UlaGua);
-#ifdef INCLUDE_CLIENT_APIS
+	#ifdef INCLUDE_CLIENT_APIS
 		sock_close(miniSocket->ssdpReqSock4);
 		sock_close(miniSocket->ssdpReqSock6);
-#endif /* INCLUDE_CLIENT_APIS */
+	#endif /* INCLUDE_CLIENT_APIS */
 		return UPNP_E_INTERNAL_ERROR;
 	}
-#ifdef INTERNAL_WEB_SERVER
+	#ifdef INTERNAL_WEB_SERVER
 	*listen_port4 = miniSocket->miniServerPort4;
 	*listen_port6 = miniSocket->miniServerPort6;
 	*listen_port6UlaGua = miniSocket->miniServerPort6UlaGua;
-#endif
+	#endif /* INTERNAL_WEB_SERVER */
 
 	return UPNP_E_SUCCESS;
 }
 
-int StopMiniServer()
+int StopMiniServer(void)
 {
 	char errorBuffer[ERROR_BUFFER_LEN];
 	socklen_t socklen = sizeof(struct sockaddr_in);
@@ -1140,7 +1480,7 @@ int StopMiniServer()
 	}
 	while (gMServState != (MiniServerState)MSERV_IDLE) {
 		ssdpAddr.sin_family = (sa_family_t)AF_INET;
-		ssdpAddr.sin_addr.s_addr = inet_addr("127.0.0.1");
+		inet_pton(AF_INET, "127.0.0.1", &ssdpAddr.sin_addr);
 		ssdpAddr.sin_port = htons(miniStopSockPort);
 		sendto(sock,
 			buf,
