@@ -46,7 +46,6 @@
 
 	#include "ThreadPool.h"
 	#include "httpparser.h"
-	#include "httpreadwrite.h"
 	#include "membuffer.h"
 	#include "miniserver.h"
 	#include "sock.h"
@@ -54,7 +53,7 @@
 
 	#include <stdio.h>
 
-	#include "posix_overwrites.h"
+	#include "posix_overwrites.h" // IWYU pragma: keep
 
 	#define MAX_TIME_TOREAD 45
 
@@ -65,7 +64,7 @@ SOCKET gSsdpReqSocket6 = INVALID_SOCKET;
 		#endif /* UPNP_ENABLE_IPV6 */
 	#endif	       /* INCLUDE_CLIENT_APIS */
 
-void RequestHandler();
+void RequestHandler(void);
 
 enum Listener
 {
@@ -125,7 +124,7 @@ int AdvertiseAndReply(int AdFlag,
 		AdFlag);
 
 	/* Use a read lock */
-	HandleReadLock();
+	HandleReadLock(__FILE__, __LINE__);
 	if (GetHandleInfo(Hnd, &SInfo) != HND_DEVICE) {
 		retVal = UPNP_E_INVALID_HANDLE;
 		goto end_function;
@@ -534,7 +533,7 @@ end_function:
 		__FILE__,
 		__LINE__,
 		"Exiting AdvertiseAndReply.\n");
-	HandleUnlock();
+	HandleUnlock(__FILE__, __LINE__);
 
 	return retVal;
 }
@@ -750,7 +749,7 @@ static UPNP_INLINE int start_event_handler(
 				SSDP,
 				__FILE__,
 				__LINE__,
-				"SSDP recvd bad msg code = %d\n",
+				"SSDP recvd bad msg code = %u\n",
 				status);
 			/* ignore bad msg, or not enuf mem */
 			goto error_handler;
@@ -761,7 +760,7 @@ static UPNP_INLINE int start_event_handler(
 			SSDP,
 			__FILE__,
 			__LINE__,
-			"SSDP recvd bad msg code = %d\n",
+			"SSDP recvd bad msg code = %u\n",
 			status);
 
 		goto error_handler;
@@ -805,7 +804,7 @@ static void ssdp_event_handler_thread(
 	free_ssdp_event_handler_data(data);
 }
 
-void readFromSSDPSocket(SOCKET socket)
+int readFromSSDPSocket(SOCKET socket)
 {
 	char *requestBuf = NULL;
 	char staticBuf[BUFSIZE];
@@ -896,8 +895,11 @@ void readFromSSDPSocket(SOCKET socket)
 			if (ThreadPoolAdd(&gRecvThreadPool, &job, NULL) != 0)
 				free_ssdp_event_handler_data(data);
 		}
-	} else
-		free_ssdp_event_handler_data(data);
+		return 0;
+	}
+	free_ssdp_event_handler_data(data);
+
+	return (byteReceived < 0) ? -1 : 0;
 }
 
 /*!
@@ -932,7 +934,7 @@ static int create_ssdp_sock_v4(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEADDR,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -950,7 +952,7 @@ static int create_ssdp_sock_v4(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEPORT,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -994,18 +996,12 @@ static int create_ssdp_sock_v4(
 	 * windows does not recognize the latter.
 	 */
 	memset((void *)&ssdpMcastAddr, 0, sizeof ssdpMcastAddr);
-	#ifdef _WIN32
-	inet_pton(AF_INET, (PCSTR)gIF_IPV4, &ssdpMcastAddr.imr_interface);
-	inet_pton(AF_INET, (PCSTR)SSDP_IP, &ssdpMcastAddr.imr_multiaddr);
-	#else
-	ssdpMcastAddr.imr_interface.s_addr = inet_addr(gIF_IPV4);
-	/* ssdpMcastAddr.imr_address.s_addr = inet_addr(gIF_IPV4); */
-	ssdpMcastAddr.imr_multiaddr.s_addr = inet_addr(SSDP_IP);
-	#endif
+	inet_pton(AF_INET, gIF_IPV4, &ssdpMcastAddr.imr_interface);
+	inet_pton(AF_INET, SSDP_IP, &ssdpMcastAddr.imr_multiaddr);
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IP,
 		IP_ADD_MEMBERSHIP,
-		(char *)&ssdpMcastAddr,
+		(OPTION_VALUE_CAST)&ssdpMcastAddr,
 		sizeof(struct ip_mreq));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1021,15 +1017,11 @@ static int create_ssdp_sock_v4(
 	}
 	/* Set multicast interface. */
 	memset((void *)&addr, 0, sizeof(struct in_addr));
-	#ifdef _WIN32
-	inet_pton(AF_INET, (PCSTR)gIF_IPV4, &addr);
-	#else
-	addr.s_addr = inet_addr(gIF_IPV4);
-	#endif
+	inet_pton(AF_INET, gIF_IPV4, &addr);
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IP,
 		IP_MULTICAST_IF,
-		(char *)&addr,
+		(OPTION_VALUE_CAST)&addr,
 		sizeof addr);
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1043,12 +1035,16 @@ static int create_ssdp_sock_v4(
 		/* This is probably not a critical error, so let's continue. */
 	}
 	/* result is not checked becuase it will fail in WinMe and Win9x. */
-	setsockopt(*ssdpSock, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
+	setsockopt(*ssdpSock,
+		IPPROTO_IP,
+		IP_MULTICAST_TTL,
+		(OPTION_VALUE_CAST)&ttl,
+		sizeof(ttl));
 	onOff = 1;
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_BROADCAST,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1096,8 +1092,11 @@ static int create_ssdp_sock_reqv4(
 			errorBuffer);
 		return UPNP_E_OUTOF_SOCKET;
 	}
-	setsockopt(
-		*ssdpReqSock, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
+	setsockopt(*ssdpReqSock,
+		IPPROTO_IP,
+		IP_MULTICAST_TTL,
+		(OPTION_VALUE_CAST)&ttl,
+		sizeof(ttl));
 	/* just do it, regardless if fails or not. */
 	sock_make_no_blocking(*ssdpReqSock);
 
@@ -1136,7 +1135,7 @@ static int create_ssdp_sock_v6(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEADDR,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1154,7 +1153,7 @@ static int create_ssdp_sock_v6(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEPORT,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1172,7 +1171,7 @@ static int create_ssdp_sock_v6(
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IPV6,
 		IPV6_V6ONLY,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1200,7 +1199,7 @@ static int create_ssdp_sock_v6(
 			SSDP,
 			__FILE__,
 			__LINE__,
-			"Error in bind(), addr=%s, index=%d, port=%d: %s\n",
+			"Error in bind(), addr=%s, index=%u, port=%d: %s\n",
 			gIF_IPV6,
 			gIF_INDEX,
 			SSDP_PORT,
@@ -1213,7 +1212,7 @@ static int create_ssdp_sock_v6(
 			SSDP,
 			__FILE__,
 			__LINE__,
-			"Error in bind(), addr=%s, index=%d, port=%d: %d\n",
+			"Error in bind(), addr=%s, index=%u, port=%d: %d\n",
 			gIF_IPV6,
 			gIF_INDEX,
 			SSDP_PORT,
@@ -1229,11 +1228,11 @@ static int create_ssdp_sock_v6(
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IPV6,
 		IPV6_JOIN_GROUP,
-		(char *)&ssdpMcastAddr,
+		(OPTION_VALUE_CAST)&ssdpMcastAddr,
 		sizeof(ssdpMcastAddr));
 	if (ret == -1) {
-		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
 		uint32_t *p = (uint32_t *)&ssdpMcastAddr.ipv6mr_multiaddr;
+		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
 		UpnpPrintf(UPNP_CRITICAL,
 			SSDP,
 			__FILE__,
@@ -1260,7 +1259,7 @@ static int create_ssdp_sock_v6(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_BROADCAST,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1316,7 +1315,7 @@ static int create_ssdp_sock_v6_ula_gua(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEADDR,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1334,7 +1333,7 @@ static int create_ssdp_sock_v6_ula_gua(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_REUSEPORT,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1352,7 +1351,7 @@ static int create_ssdp_sock_v6_ula_gua(
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IPV6,
 		IPV6_V6ONLY,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1392,7 +1391,7 @@ static int create_ssdp_sock_v6_ula_gua(
 	ret = setsockopt(*ssdpSock,
 		IPPROTO_IPV6,
 		IPV6_JOIN_GROUP,
-		(char *)&ssdpMcastAddr,
+		(OPTION_VALUE_CAST)&ssdpMcastAddr,
 		sizeof(ssdpMcastAddr));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1410,7 +1409,7 @@ static int create_ssdp_sock_v6_ula_gua(
 	ret = setsockopt(*ssdpSock,
 		SOL_SOCKET,
 		SO_BROADCAST,
-		(char *)&onOff,
+		(OPTION_VALUE_CAST)&onOff,
 		sizeof(onOff));
 	if (ret == -1) {
 		strerror_r(errno, errorBuffer, ERROR_BUFFER_LEN);
@@ -1465,7 +1464,7 @@ static int create_ssdp_sock_reqv6(
 	setsockopt(*ssdpReqSock,
 		IPPROTO_IPV6,
 		IPV6_MULTICAST_HOPS,
-		&hops,
+		(OPTION_VALUE_CAST)&hops,
 		sizeof(hops));
 	/* just do it, regardless if fails or not. */
 	sock_make_no_blocking(*ssdpReqSock);
@@ -1491,7 +1490,7 @@ int get_ssdp_sockets(MiniServerSockArray *out)
 		gSsdpReqSocket4 = out->ssdpReqSock4;
 	} else
 		out->ssdpReqSock4 = INVALID_SOCKET;
-			/* Create the IPv6 socket for SSDP REQUESTS */
+		/* Create the IPv6 socket for SSDP REQUESTS */
 		#ifdef UPNP_ENABLE_IPV6
 	if (strlen(gIF_IPV6) > (size_t)0) {
 		retVal = create_ssdp_sock_reqv6(&out->ssdpReqSock6);
